import os
import mmh3
import mmap


class BloomFilter:
    """
    Simple bloom filter implementation capable of roughly 400K lookups/s.

    BBOT uses bloom filters in scenarios like DNS brute-forcing, where it's useful to keep track
    of which mutations have been tried so far.

    A 100-megabyte bloom filter (800M bits) can store 10M entries with a .01% false-positive rate.
    A python hash is 36 bytes. So if you wanted to store these in a set, this would take up
    36 * 10M * 2 (key+value) == 720 megabytes. So we save roughly 7 times the space.
    """

    def __init__(self, size=8000000):
        self.size = size  # total bits
        self.byte_size = (size + 7) // 8  # calculate byte size needed for the given number of bits

        # Create an anonymous mmap region, compatible with both Windows and Unix
        if os.name == "nt":  # Windows
            # -1 indicates an anonymous memory map in Windows
            self.mmap_file = mmap.mmap(-1, self.byte_size)
        else:  # Unix/Linux
            # Use MAP_ANONYMOUS along with MAP_SHARED
            self.mmap_file = mmap.mmap(-1, self.byte_size, prot=mmap.PROT_WRITE, flags=mmap.MAP_ANON | mmap.MAP_SHARED)

        self.clear_all_bits()

    def add(self, item):
        for hash_value in self._hashes(item):
            index = hash_value // 8
            position = hash_value % 8
            current_byte = self.mmap_file[index]
            self.mmap_file[index] = current_byte | (1 << position)

    def check(self, item):
        for hash_value in self._hashes(item):
            index = hash_value // 8
            position = hash_value % 8
            current_byte = self.mmap_file[index]
            if not (current_byte & (1 << position)):
                return False
        return True

    def clear_all_bits(self):
        self.mmap_file.seek(0)
        # Write zeros across the entire mmap length
        self.mmap_file.write(b"\x00" * self.byte_size)

    def _hashes(self, item):
        if not isinstance(item, bytes):
            if not isinstance(item, str):
                item = str(item)
            item = item.encode("utf-8")
        return [abs(hash(item)) % self.size, abs(mmh3.hash(item)) % self.size, abs(self._fnv1a_hash(item)) % self.size]

    def _fnv1a_hash(self, data):
        hash = 0x811C9DC5  # 2166136261
        for byte in data:
            hash ^= byte
            hash = (hash * 0x01000193) % 2**32  # 16777619
        return hash

    def close(self):
        """Explicitly close the memory-mapped file."""
        self.mmap_file.close()

    def __del__(self):
        try:
            self.close()
        except Exception:
            pass

    def __contains__(self, item):
        return self.check(item)



import pytest
import ipaddress
from bbot.errors import ValidationError
from bbot.core.event.helpers import EventSeed


def test_event_seeds():
    # DNS_NAME
    dns_seed = EventSeed("evilcOrp.com.")
    assert dns_seed.type == "DNS_NAME"
    assert dns_seed.data == "evilcorp.com"
    assert dns_seed.host == "evilcorp.com"
    assert dns_seed.input == "evilcorp.com"
    assert dns_seed._target_type == "TARGET"

    # IP_ADDRESS (IPv4)
    ipv4_seed = EventSeed("192.168.1.1")
    assert ipv4_seed.type == "IP_ADDRESS"
    assert ipv4_seed.data == "192.168.1.1"
    assert ipv4_seed.host == ipaddress.ip_address("192.168.1.1")
    assert ipv4_seed.input == "192.168.1.1"

    # Test various IPv6 formats
    ipv6_formats = [
        "2001:db8::ff00:42:8329",  # Standard format
        "2001:0db8:0000:0000:0000:ff00:0042:8329",  # Full format
        "2001:db8:0:0:0:ff00:42:8329",  # Mixed format
        "::1",  # Loopback
        "::ffff:192.168.1.1",  # IPv4-mapped
        "2001:db8::",  # Subnet prefix
        "fe80::1ff:fe23:4567:890a",  # Link-local
    ]

    # IP_ADDRESS (IPv6)
    for ipv6 in ipv6_formats:
        ipv6_seed = EventSeed(ipv6)
        normalized_ipv6 = str(ipaddress.IPv6Address(ipv6))
        assert ipv6_seed.type == "IP_ADDRESS"
        assert ipv6_seed.data == normalized_ipv6
        assert ipv6_seed.host == ipaddress.ip_address(ipv6)
        assert ipv6_seed.input == normalized_ipv6

    # IP_RANGE (IPv4)
    ipv4_range_seed = EventSeed("192.168.1.1/24")
    assert ipv4_range_seed.type == "IP_RANGE"
    assert ipv4_range_seed.data == "192.168.1.0/24"
    assert ipv4_range_seed.host == ipaddress.ip_network("192.168.1.0/24")
    assert ipv4_range_seed.input == "192.168.1.0/24"

    # IP_RANGE (IPv6)
    ipv6_range_seed = EventSeed("2001:db8::ff00:42:8329/64")
    assert ipv6_range_seed.type == "IP_RANGE"
    assert ipv6_range_seed.data == "2001:db8::/64"
    assert ipv6_range_seed.host == ipaddress.ip_network("2001:db8::/64")
    assert ipv6_range_seed.input == "2001:db8::/64"

    # OPEN_TCP_PORT (DNS)
    open_port_dns_seed = EventSeed("evilcOrp.com:80")
    assert open_port_dns_seed.type == "OPEN_TCP_PORT"
    assert open_port_dns_seed.data == "evilcorp.com:80"
    assert open_port_dns_seed.host == "evilcorp.com"
    assert open_port_dns_seed.port == 80
    assert open_port_dns_seed.input == "evilcorp.com:80"

    # OPEN_TCP_PORT (IPv4)
    open_port_ipv4_seed = EventSeed("192.168.1.1:80")
    assert open_port_ipv4_seed.type == "OPEN_TCP_PORT"
    assert open_port_ipv4_seed.data == "192.168.1.1:80"
    assert open_port_ipv4_seed.host == ipaddress.ip_address("192.168.1.1")
    assert open_port_ipv4_seed.port == 80
    assert open_port_ipv4_seed.input == "192.168.1.1:80"

    # OPEN_TCP_PORT (IPv6)
    open_port_ipv6_seed = EventSeed("[2001:db8::42]:80")
    assert open_port_ipv6_seed.type == "OPEN_TCP_PORT"
    assert open_port_ipv6_seed.data == "[2001:db8::42]:80"
    assert open_port_ipv6_seed.host == ipaddress.ip_address("2001:db8::42")
    assert open_port_ipv6_seed.port == 80
    assert open_port_ipv6_seed.input == "[2001:db8::42]:80"

    # URL (DNS_NAME)
    url_dns_seed = EventSeed("http://evilcOrp.com./index.html?a=b#c")
    assert url_dns_seed.type == "URL_UNVERIFIED"
    assert url_dns_seed.data == "http://evilcorp.com/index.html?a=b"
    assert url_dns_seed.host == "evilcorp.com"
    assert url_dns_seed.port == 80
    assert url_dns_seed.input == "http://evilcorp.com/index.html?a=b"

    # URL (IPv4)
    url_ipv4_seed = EventSeed("https://192.168.1.1/index.html?a=b#c")
    assert url_ipv4_seed.type == "URL_UNVERIFIED"
    assert url_ipv4_seed.data == "https://192.168.1.1/index.html?a=b"
    assert url_ipv4_seed.host == ipaddress.ip_address("192.168.1.1")
    assert url_ipv4_seed.port == 443
    assert url_ipv4_seed.input == "https://192.168.1.1/index.html?a=b"

    # URL (IPv6)
    url_ipv6_seed = EventSeed("https://[2001:db8::42]:8080/index.html?a=b#c")
    assert url_ipv6_seed.type == "URL_UNVERIFIED"
    assert url_ipv6_seed.data == "https://[2001:db8::42]:8080/index.html?a=b"
    assert url_ipv6_seed.host == ipaddress.ip_address("2001:db8::42")
    assert url_ipv6_seed.port == 8080
    assert url_ipv6_seed.input == "https://[2001:db8::42]:8080/index.html?a=b"

    # EMAIL_ADDRESS
    email_seed = EventSeed("john.doe@evilcOrp.com")
    assert email_seed.type == "EMAIL_ADDRESS"
    assert email_seed.data == "john.doe@evilcorp.com"
    assert email_seed.host == "evilcorp.com"
    assert email_seed.port == None
    assert email_seed.input == "john.doe@evilcorp.com"

    email_seed_ipv4 = EventSeed("john.doe@192.168.1.1:80")
    assert email_seed_ipv4.type == "EMAIL_ADDRESS"
    assert email_seed_ipv4.data == "john.doe@192.168.1.1:80"
    assert email_seed_ipv4.host == ipaddress.ip_address("192.168.1.1")
    assert email_seed_ipv4.port == 80
    assert email_seed_ipv4.input == "john.doe@192.168.1.1:80"

    # ORG_STUB
    org_stub_seed = EventSeed("ORG:evilcorp")
    assert org_stub_seed.type == "ORG_STUB"
    assert org_stub_seed.data == "evilcorp"
    assert org_stub_seed.host == None
    assert org_stub_seed.input == "ORG_STUB:evilcorp"

    # USERNAME
    username_seed = EventSeed("USER:john.doe")
    assert username_seed.type == "USERNAME"
    assert username_seed.data == "john.doe"
    assert username_seed.host == None
    assert username_seed.input == "USERNAME:john.doe"

    # FILESYSTEM
    filesystem_seed = EventSeed("FILE:/home/john/documents")
    assert filesystem_seed.type == "FILESYSTEM"
    assert filesystem_seed.data == {"path": "/home/john/documents"}
    assert filesystem_seed.host == None
    assert filesystem_seed.input == "FILESYSTEM:/home/john/documents"

    # MOBILE_APP
    mobile_app_seed = EventSeed("APK:https://play.google.com/store/apps/details?id=com.evilcorp.app")
    assert mobile_app_seed.type == "MOBILE_APP"
    assert mobile_app_seed.data == {"url": "https://play.google.com/store/apps/details?id=com.evilcorp.app"}
    assert mobile_app_seed.host == None
    assert mobile_app_seed.input == "MOBILE_APP:https://play.google.com/store/apps/details?id=com.evilcorp.app"

    with pytest.raises(ValidationError):
        EventSeed("INVALID:INVALID")

    with pytest.raises(ValidationError):
        EventSeed("^@#$^@#$")

    # BLACKLIST_REGEX
    blacklist_regex_seed = EventSeed("RE:evil[0-9]{3}")
    assert blacklist_regex_seed.type == "BLACKLIST_REGEX"
    assert blacklist_regex_seed.data == "evil[0-9]{3}"
    assert blacklist_regex_seed.host == None
    assert blacklist_regex_seed.input == "REGEX:evil[0-9]{3}"
    assert blacklist_regex_seed._target_type == "BLACKLIST"



from .base import ModuleTestBase


class TestViewDNS(ModuleTestBase):
    async def setup_after_prep(self, module_test):
        module_test.httpx_mock.add_response(
            url="https://viewdns.info/reversewhois/?q=blacklanternsecurity.com",
            text=web_body,
        )

    def check(self, module_test, events):
        assert any(e.data == "hyperloop.com" and "affiliate" in e.tags for e in events), (
            "Failed to detect affiliate domain"
        )


web_body = """<html>
   <head>
      <meta name="google-site-verification" content="DUBJr87ZeILnfEKxhntAq9XPSZCa2mb3W4FAwXjKpyk" />
      <title>Reverse Whois Lookup - ViewDNS.info</title>
      <meta keywords="viewdns, dns, info, reverse ip, pagerank, portscan, port scan, lookup, records, whois, ipwhois, dnstools, web hosting, hosting, traceroute, dns report, dnsreport, ip location, ip location finder, spam, spam database, dnsbl, propagation, dns propagation checker, checker, china, chinese, firewall, great firewall, is my site down, is site down, site down, down, dns propagate">
      <meta description="This free tool will allow you to find domain names owned by an individual person or company.  Simply enter the email address or name of the person or company to find other domains registered using those same details.">
      <!-- form validation -->
   </head>
   <body bgcolor="#ededed">
      <font face="Verdana">
      <table width="1000" align="center">
         <tr height="62">
            <td align="left" width="400"><a href="/"><img src="/images/viewdns_logo.gif" border="0" width="399" height="42.5" alt="ViewDNS.info - Your one source for DNS related tools!"></a></td>
            <td align="center" valign="middle">
               <script async src="//pagead2.googlesyndication.com/pagead/js/adsbygoogle.js"></script>
               <!-- ViewDNS 468x60 -->
               <ins class="adsbygoogle"
                  style="display:inline-block;width:468px;height:60px"
                  data-ad-client="ca-pub-7431844373287199"
                  data-ad-slot="1039512844"></ins>
               <script>
                  (adsbygoogle = window.adsbygoogle || []).push({});
               </script>
            </td>
         </tr>
      </table>
      <!-- tabs -->
      <!--<div id="header" width="1000" align="center"> -->
      <div id="header">
         <table width="1000" align="center" cellspacing="0" cellpadding="0">
            <tbody>
               <tr>
                  <td>
                     <ul>
                        <li id="selected"><a href="https://viewdns.info/">Tools</a></li>
                        <li id="notselected"><a href="https://viewdns.info/api/">API</a></li>
                        <li id="notselected"><a href="https://viewdns.info/research/">Research</a></li>
                        <li id="notselected"><a href="https://viewdns.info/data/">Data</a></li>
                     </ul>
                  </td>
               </tr>
            </tbody>
         </table>
      </div>
      <!--</div>-->
      <!-- end tabs-->
      <!--<div>-->
      <table width="1000" bgcolor="#FFFFFF" style="border: 1px solid #CCCCCC; padding: 5px" align="center" id="null">
         <tr></tr>
         <tr>
            <td>
               <font size="2">
                  <a href="/" style="color: #00721e;">ViewDNS.info</a> > <a href="/" style="color: #00721e;">Tools</a> >
                  <H1 style="font-size: 16; display: inline;">Reverse Whois Lookup</H1>
                  <br><br>This free tool will allow you to find domain names owned by an individual person or company.  Simply enter the email address or name of the person or company to find other domains registered using those same details. <a href="#" onclick="javascript:document.getElementById('faq').style.visibility = 'visible'; document.getElementById('faq').style.display = 'block';"  style="color: #00721e;">FAQ</a>.<br><br>
                  <div id="faq" style="visibility: hidden; display: none;"><u><b>Frequently Asked Questions</b></u><br>Q. Will this tool return results for all domains including ccTLD's?<br>A. Unfortunately no.  Whilst we do our best to ensure our data is as complete as possible, we are not able to return results for all ccTLD's.  Due to a number of technical limitations with whois data, the results from any Reverse Whois tool should not be considered as exhaustive.<br><br>Q. Is your data live?<br>A. Our data is not live.  We do our best to update the data as often as possible with daily updates for selected TLD's and quarterly updates for others.<br><br>Q. How do I see all records for a specific person/company rather than the limited number you show on your site?<br>A. Please <a href="mailto:feedback@viewdns.info?subject=Reverse Whois" style="color: #00721e;">email us</a> with your request and we'll see what we can do for you.<br><br></div>
                  <form name="reversewhois" action="" method="GET" >Registrant Name or Email Address: <br>
               </font>
               <input name="q" type="text" size="30"><input type="submit" value="GO"></form>
            </td>
         </tr>
         <tr>
            <td>
               <font size="2" face="Courier">Reverse Whois results for blacklanternsecurity.com<br>==============<br><br>There are 20 domains that matched this search query.<br>These are listed below:<br><br>
               <table border="1">
                  <tr>
                     <td>hyperloop.com</td>
                     <td>2003-12-04</td>
                     <td>NETWORK SOLUTIONS, LLC.</td>
                  </tr>
               </table>
               <br>
            </td>
         </tr>
         <tr></tr>
      </table>
      <!--</div>-->
      <table width="1000" align="center" border="0">
         <tr align="center">
            <td align="center">
               <script async src="//pagead2.googlesyndication.com/pagead/js/adsbygoogle.js"></script>
               <!-- viewdns-bottom-linkunit -->
               <ins class="adsbygoogle"
                  style="display:inline-block;width:728px;height:15px"
                  data-ad-client="ca-pub-7431844373287199"
                  data-ad-slot="9102586825"></ins>
               <script>
                  (adsbygoogle = window.adsbygoogle || []).push({});
               </script>
               <br /><br />
               <!-- fb -->
               <div id="fb-root"></div>
               <script>(function(d, s, id) {
                  var js, fjs = d.getElementsByTagName(s)[0];
                  if (d.getElementById(id)) return;
                  js = d.createElement(s); js.id = id;
                  js.src = "//connect.facebook.net/en_US/sdk.js#xfbml=1&appId=187997344602848&version=v2.0";
                  fjs.parentNode.insertBefore(js, fjs);
                  }(document, 'script', 'facebook-jssdk'));
               </script>
               <!-- end fb -->
               <a href="https://twitter.com/viewdns" class="twitter-follow-button" data-show-count="false" align="center">Follow @viewdns</a>
               <script>!function(d,s,id){var js,fjs=d.getElementsByTagName(s)[0];if(!d.getElementById(id)){js=d.createElement(s);js.id=id;js.src="//platform.twitter.com/widgets.js";fjs.parentNode.insertBefore(js,fjs);}}(document,"script","twitter-wjs");</script>
               <div class="fb-like" data-href="https://www.facebook.com/viewdns" data-layout="button" data-action="like" data-show-faces="true" data-share="true"></div>
               <br />
               <font size="1">All content &copy; 2023 ViewDNS.info<br><a href="mailto:feedback@viewdns.info?subject=Feedback" style="color: #00721e;">Feedback / Suggestions / Contact Us</a>&nbsp-&nbsp<a href="https://viewdns.info/privacy.php" style="color: #00721e;">Privacy Policy</a></font>
            </td>
         </tr>
      </table>
      <br />
      <table width="731" align="center" border="0">
         <tr align="center">
            <td align="center">
               <!--INLINE-->
               <script type="text/javascript"><!--
                  google_ad_client = "ca-pub-7431844373287199";
                  /* ViewDNS 728x90 */
                  google_ad_slot = "2958648842";
                  google_ad_width = 728;
                  google_ad_height = 90;
                  google_page_url="http://viewdns.info";
                  //-->
               </script>
               <script type="text/javascript"
                  src="//pagead2.googlesyndication.com/pagead/show_ads.js"></script>
            </td>
         </tr>
      </table>
      <center>
         <br>
         <br>
      </center>
      <!-- page generated in 0.41837406158447 seconds -->
   </body>
</html>"""



from pathlib import Path
from .base import ModuleTestBase
from bbot.test.bbot_fixtures import bbot_test_dir


class TestGitDumper_Dirlisting(ModuleTestBase):
    targets = [
        "http://127.0.0.1:8888/test",
    ]

    modules_overrides = ["git", "gitdumper", "httpx"]
    config_overrides = {"modules": {"gitdumper": {"output_folder": str(bbot_test_dir / "test_output")}}}

    index_html = """<html>
        <head>
            <title>Index of /.git</title>
        </head>
        <body>
            <h1>Index of /.git</h1>
            <table>
                <tr><th>Name</th><th>Size</th></tr>
                <tr><td><a href='/test/.git/branches/'>&lt;branches&gt;</a></td><td></td></tr>
                <tr><td><a href='/test/.git/COMMIT_EDITMSG'>COMMIT_EDITMSG</a></td><td>157B</td></tr>
                <tr><td><a href='/test/.git/config'>config</a></td><td>157B</td></tr>
                <tr><td><a href='/test/.git/description'>description</a></td><td>73B</td></tr>
                <tr><td><a href='/test/.git/HEAD'>HEAD</a></td><td>23B</td></tr>
                <tr><td><a href='/test/.git/hooks/'>&lt;hooks&gt;</a></td><td></td></tr>
                <tr><td><a href='/test/.git/info/'>&lt;info&gt;</a></td><td></td></tr>
                <tr><td><a href='/test/.git/objects/'>&lt;objects&gt;</a></td><td></td></tr>
                <tr><td><a href='/test/.git/index'>index</a></td><td></td></tr>
                <tr><td><a href='/test/.git/refs/'>&lt;refs&gt;</a></td><td></td></tr>
                <tr><td><a href='/test/.git/logs/'>&lt;logs&gt;</a></td><td></td></tr>
            </table>
        </body>
    </html>"""

    info_index = """<html>
        <head>
            <title>Index of /.git/info</title>
        </head>
        <body>
            <h1>Index of /.git/info</h1>
            <table>
                <tr><th>Name</th><th>Size</th></tr>
                <tr><td><a href='../'>[..]</a></td><td></td></tr>
                <tr><td><a href='/test/.git/info/exclude'>exclude</a></td><td>240B</td></tr>
                <tr><td><a href='http://exclude.com/excludeme'>excludeme</a></td><td>0B</td></tr>
            </table>
        </body>
    </html>"""

    objects_index = """<html>
        <head>
            <title>Index of /.git/objects</title>
        </head>
        <body>
            <h1>Index of /.git/objects</h1>
            <table>
                <tr><th>Name</th><th>Size</th></tr>
                <tr><td><a href='../'>[..]</a></td><td></td></tr>
                <tr><td><a href='/test/.git/objects/05/'>&lt;05&gt;</a></td><td></td></tr>
                <tr><td><a href='/test/.git/objects/34/'>&lt;34&gt;</a></td><td></td></tr>
                <tr><td><a href='/test/.git/objects/c2/'>&lt;c2&gt;</a></td><td></td></tr>
                <tr><td><a href='/test/.git/objects/pack/'>&lt;pack&gt;</a></td><td></td></tr>
                <tr><td><a href='/test/.git/objects/info/'>&lt;info&gt;</a></td><td></td></tr>
            </table>
        </body>
    </html>"""

    objects_o5_index = """<html>
        <head>
            <title>Index of /.git/objects/05</title>
        </head>
        <body>
            <h1>Index of /.git/objects/05</h1>
            <table>
                <tr><th>Name</th><th>Size</th></tr>
                <tr><td><a href='../'>[..]</a></td><td></td></tr>
                <tr><td><a href='/test/.git/objects/05/27e6bd2d76b45e2933183f1b506c7ac49f5872'>27e6bd2d76b45e2933183f1b506c7ac49f5872</a></td><td></td></tr>
            </table>
        </body>
    </html>"""

    objects_34_index = """<html>
        <head>
            <title>Index of /.git/objects/34</title>
        </head>
        <body>
            <h1>Index of /.git/objects/34</h1>
            <table>
                <tr><th>Name</th><th>Size</th></tr>
                <tr><td><a href='../'>[..]</a></td><td></td></tr>
                <tr><td><a href='/test/.git/objects/34/dc86f0247798892a89553e7c5c2d5aa06c2c5b'>dc86f0247798892a89553e7c5c2d5aa06c2c5b</a></td><td></td></tr>
            </table>
        </body>
    </html>"""

    objects_c2_index = """<html>
        <head>
            <title>Index of /.git/objects/c2</title>
        </head>
        <body>
            <h1>Index of /.git/objects/c2</h1>
            <table>
                <tr><th>Name</th><th>Size</th></tr>
                <tr><td><a href='../'>[..]</a></td><td></td></tr>
                <tr><td><a href='/test/.git/objects/c2/69d751b8e2fd0be0d0dc7a6437a4dce4ec0200'>69d751b8e2fd0be0d0dc7a6437a4dce4ec0200</a></td><td></td></tr>
            </table>
        </body>
    </html>"""

    refs_index = """<html>
        <head>
            <title>Index of /.git/refs</title>
        </head>
        <body>
            <h1>Index of /.git/refs</h1>
            <table>
                <tr><th>Name</th><th>Size</th></tr>
                <tr><td><a href='../'>[..]</a></td><td></td></tr>
                <tr><td><a href='/test/.git/refs/heads/'>&lt;heads&gt;</a></td><td></td></tr>
                <tr><td><a href='/test/.git/refs/tags/'>&lt;tags&gt;</a></td><td></td></tr>
            </table>
        </body>
    </html>
    """

    refs_heads_index = """<html>
        <head>
            <title>Index of /.git/refs/heads</title>
        </head>
        <body>
            <h1>Index of /.git/refs/heads</h1>
            <table>
                <tr><th>Name</th><th>Size</th></tr>
                <tr><td><a href='../'>[..]</a></td><td></td></tr>
                <tr><td><a href='/test/.git/refs/heads/master'>master</a></td><td></td></tr>
            </table>
        </body>
    </html>
    """

    logs_index = """<html>
        <head>
            <title>Index of /.git/logs</title>
        </head>
        <body>
            <h1>Index of /.git/logs</h1>
            <table>
                <tr><th>Name</th><th>Size</th></tr>
                <tr><td><a href='../'>[..]</a></td><td></td></tr>
                <tr><td><a href='/test/.git/logs/HEAD'>HEAD</a></td><td></td></tr>
                <tr><td><a href='/test/.git/logs/refs/'>&lt;tags&gt;</a></td><td></td></tr>
            </table>
        </body>
    </html>
    """

    logs_refs_index = """<html>
        <head>
            <title>Index of /.git/logs/refs</title>
        </head>
        <body>
            <h1>Index of /.git/logs/refs</h1>
            <table>
                <tr><th>Name</th><th>Size</th></tr>
                <tr><td><a href='../'>[..]</a></td><td></td></tr>
                <tr><td><a href='/test/.git/logs/refs/heads/'>&lt;heads&gt;</a></td><td></td></tr>
            </table>
        </body>
    </html>
    """

    logs_refs_heads_index = """<html>
        <head>
            <title>Index of /.git/logs/refs/heads</title>
        </head>
        <body>
            <h1>Index of /.git/logs/refs/heads</h1>
            <table>
                <tr><th>Name</th><th>Size</th></tr>
                <tr><td><a href='../'>[..]</a></td><td></td></tr>
                <tr><td><a href='/test/.git/logs/refs/heads/master'>master</a></td><td></td></tr>
            </table>
        </body>
    </html>
    """

    empty_index = """<html>
        <head>
            <title>Index of /.git/...</title>
        </head>
        <body>
            <h1>Index of /.git/...</h1>
            <table>
                <tr><th>Name</th><th>Size</th></tr>
                <tr><td><a href='../'>[..]</a></td><td></td></tr>
            </table>
        </body>
    </html>"""

    git_head = "ref: refs/heads/master"

    refs_head = "34dc86f0247798892a89553e7c5c2d5aa06c2c5b"

    logs_head = "0000000000000000000000000000000000000000 34dc86f0247798892a89553e7c5c2d5aa06c2c5b Test <test@test.com> 1738516534 +0000	commit (initial): Initial commit"

    logs_master_head = "0000000000000000000000000000000000000000 34dc86f0247798892a89553e7c5c2d5aa06c2c5b Test <test@test.com> 1738516534 +0000	commit (initial): Initial commit"

    git_description = "Unnamed repository; edit this file 'description' to name the repository."

    git_commit_editmsg = "Initial commit"

    git_config = """[core]
    repositoryformatversion = 0
    filemode = true
    bare = false
    logallrefupdates = true"""

    git_exclude = """# git ls-files --others --exclude-from=.git/info/exclude
    # Lines that start with '#' are comments.
    # For a project mostly in C, the following would be a good set of
    # exclude patterns (uncomment them if you want to use them):
    # *.[oa]
    # *~"""

    filebytes_gitindex = b"DIRC\x00\x00\x00\x02\x00\x00\x00\x01g\x9f\xbe\x04\x14\xfcb\xd1g\x9f\xbe\x04\x14\xfcb\xd1\x00\x00\x08 \x00\x04aD\x00\x00\x81\xa4\x00\x00\x03\xe8\x00\x00\x03\xe8\x00\x00\x00\x0f\x05'\xe6\xbd-v\xb4^)3\x18?\x1bPlz\xc4\x9fXr\x00\x08test.txt\x00\x00TREE\x00\x00\x00\x19\x001 0\n\xc2i\xd7Q\xb8\xe2\xfd\x0b\xe0\xd0\xdczd7\xa4\xdc\xe4\xec\x02\x00\xe8m|iw\xbb\xd6\x88;f\xdbW\x10yY\xd2\xb0G\xcfJ"
    filebytes_27e6bd2d76b45e2933183f1b506c7ac49f5872 = (
        b"x\x01K\xca\xc9OR04e\x08\xc9\xc8,V\x00\xa2D\x85\x92\xd4\xe2\x12.\x00U\xab\x07%"
    )
    filebytes_dc86f0247798892a89553e7c5c2d5aa06c2c5b = b"x\x01\x9d\x8dK\n\x021\x10D]\xe7\x14\xbd\x17\x86\xce?\x82\x88\x0b7\x9e\xc0u\xa6\xd3:\x81\xc4\xc0\x18\x99\xeb\x1b\x98\x1bX\xbbzP\xaf\xa8\xd5\x9a;\xc8\xa0\x0f}e\x06R\xee\x94\xbc\x95s`\xf5L83&L\xe4\xa33\xdaG\x93\x88\r\x13*D\x11\xbf}i+\xdcZ\x85\xc7\xc2\x1b\x97\x02\xe7\xd4\xea\xb4\xed\xe5\xfa\x89/\x9e\xa8\xd5\x0bH\xaf\x83\x95\xcej\x03G\x1c\x11\x83\x8e\xcf\xce\xff\xad\xc5\xfd\x9d{\x8e\x05v\x8d\xf8\x01\xfaF<\x05"
    filebytes_69d751b8e2fd0be0d0dc7a6437a4dce4ec0200 = b"x\x01+)JMU06c040031Q(I-.\xd1+\xa9(a`U\x7f\xb6W\xb7lK\x9c\xa6\xb1\x84\xbdt@N\xd5\x91\xf9\x11E\x00*\x05\x0e\x8c"

    async def setup_after_prep(self, module_test):
        module_test.set_expect_requests(
            expect_args={"uri": "/test/.git/"}, respond_args={"response_data": self.index_html}
        )
        module_test.set_expect_requests(
            expect_args={"uri": "/test/.git/index"}, respond_args={"response_data": self.filebytes_gitindex}
        )
        module_test.set_expect_requests(
            expect_args={"uri": "/test/.git/COMMIT_EDITMSG"}, respond_args={"response_data": self.git_commit_editmsg}
        )
        module_test.set_expect_requests(
            expect_args={"uri": "/test/.git/config"}, respond_args={"response_data": self.git_config}
        )
        module_test.set_expect_requests(
            expect_args={"uri": "/test/.git/branches/"}, respond_args={"response_data": self.empty_index}
        )
        module_test.set_expect_requests(
            expect_args={"uri": "/test/.git/description"}, respond_args={"response_data": self.git_description}
        )
        module_test.set_expect_requests(
            expect_args={"uri": "/test/.git/HEAD"}, respond_args={"response_data": self.git_head}
        )
        module_test.set_expect_requests(
            expect_args={"uri": "/test/.git/hooks/"}, respond_args={"response_data": self.empty_index}
        )
        module_test.set_expect_requests(
            expect_args={"uri": "/test/.git/info/"}, respond_args={"response_data": self.info_index}
        )
        module_test.set_expect_requests(
            expect_args={"uri": "/test/.git/info/exclude"}, respond_args={"response_data": self.git_exclude}
        )
        module_test.set_expect_requests(
            expect_args={"uri": "/test/.git/objects/"}, respond_args={"response_data": self.objects_index}
        )
        module_test.set_expect_requests(
            expect_args={"uri": "/test/.git/objects/05/"}, respond_args={"response_data": self.objects_o5_index}
        )
        module_test.set_expect_requests(
            expect_args={"uri": "/test/.git/objects/05/27e6bd2d76b45e2933183f1b506c7ac49f5872"},
            respond_args={"response_data": self.filebytes_27e6bd2d76b45e2933183f1b506c7ac49f5872},
        )
        module_test.set_expect_requests(
            expect_args={"uri": "/test/.git/objects/34/"}, respond_args={"response_data": self.objects_34_index}
        )
        module_test.set_expect_requests(
            expect_args={"uri": "/test/.git/objects/34/dc86f0247798892a89553e7c5c2d5aa06c2c5b"},
            respond_args={"response_data": self.filebytes_dc86f0247798892a89553e7c5c2d5aa06c2c5b},
        )
        module_test.set_expect_requests(
            expect_args={"uri": "/test/.git/objects/c2/"}, respond_args={"response_data": self.objects_c2_index}
        )
        module_test.set_expect_requests(
            expect_args={"uri": "/test/.git/objects/c2/69d751b8e2fd0be0d0dc7a6437a4dce4ec0200"},
            respond_args={"response_data": self.filebytes_69d751b8e2fd0be0d0dc7a6437a4dce4ec0200},
        )
        module_test.set_expect_requests(
            expect_args={"uri": "/test/.git/objects/info/"}, respond_args={"response_data": self.empty_index}
        )
        module_test.set_expect_requests(
            expect_args={"uri": "/test/.git/objects/pack/"}, respond_args={"response_data": self.empty_index}
        )
        module_test.set_expect_requests(
            expect_args={"uri": "/test/.git/refs/"}, respond_args={"response_data": self.refs_index}
        )
        module_test.set_expect_requests(
            expect_args={"uri": "/test/.git/refs/heads/"}, respond_args={"response_data": self.refs_heads_index}
        )
        module_test.set_expect_requests(
            expect_args={"uri": "/test/.git/refs/heads/master"}, respond_args={"response_data": self.refs_head}
        )
        module_test.set_expect_requests(
            expect_args={"uri": "/test/.git/refs/tags/"}, respond_args={"response_data": self.empty_index}
        )
        module_test.set_expect_requests(
            expect_args={"uri": "/test/.git/logs/"}, respond_args={"response_data": self.logs_index}
        )
        module_test.set_expect_requests(
            expect_args={"uri": "/test/.git/logs/refs/"}, respond_args={"response_data": self.logs_refs_index}
        )
        module_test.set_expect_requests(
            expect_args={"uri": "/test/.git/logs/refs/heads/"},
            respond_args={"response_data": self.logs_refs_heads_index},
        )
        module_test.set_expect_requests(
            expect_args={"uri": "/test/.git/logs/refs/heads/master"},
            respond_args={"response_data": self.logs_master_head},
        )
        module_test.set_expect_requests(
            expect_args={"uri": "/test/.git/logs/HEAD"}, respond_args={"response_data": self.logs_head}
        )

    def check(self, module_test, events):
        assert any(
            e.type == "CODE_REPOSITORY"
            and "git-directory" in e.tags
            and e.data["url"] == "http://127.0.0.1:8888/test/.git/"
            for e in events
        )
        filesystem_events = [
            e
            for e in events
            if e.type == "FILESYSTEM" and "http-127-0-0-1-8888-test-git" in e.data["path"] and "git" in e.tags
        ]
        assert 1 == len(filesystem_events), "Failed to git clone CODE_REPOSITORY"
        filesystem_event = filesystem_events[0]
        folder = Path(filesystem_event.data["path"])
        assert folder.is_dir(), "Destination folder doesn't exist"
        with open(folder / "test.txt") as f:
            content = f.read()
            assert content == "This is a test\n", "File content doesn't match"


class TestGitDumper_NoDirlisting(TestGitDumper_Dirlisting):
    async def setup_after_prep(self, module_test):
        module_test.set_expect_requests(
            expect_args={"uri": "/test/.git/index"}, respond_args={"response_data": self.filebytes_gitindex}
        )
        module_test.set_expect_requests(
            expect_args={"uri": "/test/.git/COMMIT_EDITMSG"}, respond_args={"response_data": self.git_commit_editmsg}
        )
        module_test.set_expect_requests(
            expect_args={"uri": "/test/.git/config"}, respond_args={"response_data": self.git_config}
        )
        module_test.set_expect_requests(
            expect_args={"uri": "/test/.git/description"}, respond_args={"response_data": self.git_description}
        )
        module_test.set_expect_requests(
            expect_args={"uri": "/test/.git/HEAD"}, respond_args={"response_data": self.git_head}
        )
        module_test.set_expect_requests(
            expect_args={"uri": "/test/.git/info/exclude"}, respond_args={"response_data": self.git_exclude}
        )
        module_test.set_expect_requests(
            expect_args={"uri": "/test/.git/objects/05/27e6bd2d76b45e2933183f1b506c7ac49f5872"},
            respond_args={"response_data": self.filebytes_27e6bd2d76b45e2933183f1b506c7ac49f5872},
        )
        module_test.set_expect_requests(
            expect_args={"uri": "/test/.git/objects/34/dc86f0247798892a89553e7c5c2d5aa06c2c5b"},
            respond_args={"response_data": self.filebytes_dc86f0247798892a89553e7c5c2d5aa06c2c5b},
        )
        module_test.set_expect_requests(
            expect_args={"uri": "/test/.git/objects/c2/69d751b8e2fd0be0d0dc7a6437a4dce4ec0200"},
            respond_args={"response_data": self.filebytes_69d751b8e2fd0be0d0dc7a6437a4dce4ec0200},
        )
        module_test.set_expect_requests(
            expect_args={"uri": "/test/.git/refs/heads/master"}, respond_args={"response_data": self.refs_head}
        )
        module_test.set_expect_requests(
            expect_args={"uri": "/test/.git/logs/refs/heads/master"},
            respond_args={"response_data": self.logs_master_head},
        )
        module_test.set_expect_requests(
            expect_args={"uri": "/test/.git/logs/HEAD"}, respond_args={"response_data": self.logs_head}
        )



from .base import ModuleTestBase


class TestTrickest(ModuleTestBase):
    config_overrides = {"modules": {"trickest": {"api_key": "deadbeef"}}}

    async def setup_before_prep(self, module_test):
        module_test.httpx_mock.add_response(
            url="https://api.trickest.io/solutions/v1/public/solution/a7cba1f1-df07-4a5c-876a-953f178996be/dataset",
            match_headers={"Authorization": "Token deadbeef"},
            json={},
        )
        module_test.httpx_mock.add_response(
            url="https://api.trickest.io/solutions/v1/public/solution/a7cba1f1-df07-4a5c-876a-953f178996be/view?q=hostname%20~%20%22.blacklanternsecurity.com%22&dataset_id=a0a49ca9-03bb-45e0-aa9a-ad59082ebdfc&limit=50&offset=0&select=hostname&orderby=hostname",
            match_headers={"Authorization": "Token deadbeef"},
            json={"results": [{"hostname": "asdf.blacklanternsecurity.com"}]},
        )
        module_test.httpx_mock.add_response(
            url="https://api.trickest.io/solutions/v1/public/solution/a7cba1f1-df07-4a5c-876a-953f178996be/view?q=hostname%20~%20%22.blacklanternsecurity.com%22&dataset_id=a0a49ca9-03bb-45e0-aa9a-ad59082ebdfc&limit=50&offset=50&select=hostname&orderby=hostname",
            match_headers={"Authorization": "Token deadbeef"},
            json={"results": [{"hostname": "www.blacklanternsecurity.com"}]},
        )

    def check(self, module_test, events):
        assert any(e.data == "asdf.blacklanternsecurity.com" for e in events), "Failed to detect subdomain"
        assert any(e.data == "www.blacklanternsecurity.com" for e in events), "Failed to detect subdomain"



from ..bbot_fixtures import *  # noqa: F401


@pytest.mark.asyncio
async def test_config(bbot_scanner):
    config = OmegaConf.create(
        {
            "plumbus": "asdf",
            "speculate": True,
            "modules": {
                "ipneighbor": {"test_option": "ipneighbor"},
                "python": {"test_option": "asdf"},
                "speculate": {"test_option": "speculate"},
            },
        }
    )
    scan1 = bbot_scanner("127.0.0.1", modules=["ipneighbor"], config=config)
    await scan1.load_modules()
    assert scan1.config.web.user_agent == "BBOT Test User-Agent"
    assert scan1.config.plumbus == "asdf"
    assert scan1.modules["ipneighbor"].config.test_option == "ipneighbor"
    assert scan1.modules["python"].config.test_option == "asdf"
    assert scan1.modules["speculate"].config.test_option == "speculate"

    await scan1._cleanup()



#!/usr/bin/env python3

import os
import re
import json
import yaml
from pathlib import Path

from bbot import Preset
from bbot.core.modules import MODULE_LOADER


DEFAULT_PRESET = Preset()

os.environ["BBOT_TABLE_FORMAT"] = "github"


# Make a regex pattern which will match any group of non-space characters that include a blacklisted character
blacklist_chars = ["<", ">"]
blacklist_re = re.compile(r"\|([^|]*[" + re.escape("".join(blacklist_chars)) + r"][^|]*)\|")

bbot_code_dir = Path(__file__).parent.parent.parent


def gen_chord_data():
    # This function generates the dataset for the chord graph in the documentation
    #  showing relationships between BBOT modules and their consumed/produced event types
    preloaded_mods = sorted(MODULE_LOADER.preloaded().items(), key=lambda x: x[0])

    entity_lookup_table = {}
    rels = []
    entities = {}
    entity_counter = 1

    def add_entity(entity, parent_id):
        if entity not in entity_lookup_table:
            nonlocal entity_counter
            e_id = entity_counter
            entity_counter += 1
            entity_lookup_table[entity] = e_id
            entity_lookup_table[e_id] = entity
            entities[e_id] = {"id": e_id, "name": entity, "parent": parent_id, "consumes": [], "produces": []}
        return entity_lookup_table[entity]

    # create entities for all the modules and event types
    for module, preloaded in preloaded_mods:
        watched = [e for e in preloaded["watched_events"] if e != "*"]
        produced = [e for e in preloaded["produced_events"] if e != "*"]
        if watched or produced:
            m_id = add_entity(module, 99999999)
            for event_type in watched:
                e_id = add_entity(event_type, 88888888)
                entities[m_id]["consumes"].append(e_id)
                entities[e_id]["consumes"].append(m_id)
            for event_type in produced:
                e_id = add_entity(event_type, 88888888)
                entities[m_id]["produces"].append(e_id)
                entities[e_id]["produces"].append(m_id)

    def add_rel(incoming, outgoing, t):
        if incoming == "*" or outgoing == "*":
            return
        i_id = entity_lookup_table[incoming]
        o_id = entity_lookup_table[outgoing]
        rels.append({"source": i_id, "target": o_id, "type": t})

    # create all the module <--> event type relationships
    for module, preloaded in preloaded_mods:
        for event_type in preloaded["watched_events"]:
            add_rel(module, event_type, "consumes")
        for event_type in preloaded["produced_events"]:
            add_rel(event_type, module, "produces")

    # write them to JSON files
    data_dir = Path(__file__).parent.parent.parent / "docs" / "data" / "chord_graph"
    data_dir.mkdir(parents=True, exist_ok=True)
    entity_file = data_dir / "entities.json"
    rels_file = data_dir / "rels.json"

    entities = [
        {"id": 77777777, "name": "root"},
        {"id": 99999999, "name": "module", "parent": 77777777},
        {"id": 88888888, "name": "event_type", "parent": 77777777},
    ] + sorted(entities.values(), key=lambda x: x["name"])

    with open(entity_file, "w") as f:
        json.dump(entities, f, indent=4)

    with open(rels_file, "w") as f:
        json.dump(rels, f, indent=4)


def homedir_collapseuser(f):
    f = Path(f)
    home_dir = Path.home()
    if f.is_relative_to(home_dir):
        return Path("~") / f.relative_to(home_dir)
    return f


def enclose_tags(text):
    # Use re.sub() to replace matched words with the same words enclosed in backticks
    result = blacklist_re.sub(r"|`\1`|", text)
    return result


def find_replace_markdown(content, keyword, replace):
    begin_re = re.compile(r"<!--\s*" + keyword + r"\s*-->", re.I)
    end_re = re.compile(r"<!--\s*END\s+" + keyword + r"\s*-->", re.I)

    begin_match = begin_re.search(content)
    end_match = end_re.search(content)

    new_content = str(content)
    if begin_match and end_match:
        start_index = begin_match.span()[-1] + 1
        end_index = end_match.span()[0] - 1
        new_content = new_content[:start_index] + enclose_tags(replace) + new_content[end_index:]
    return new_content


def find_replace_file(file, keyword, replace):
    with open(file) as f:
        content = f.read()
        new_content = find_replace_markdown(content, keyword, replace)
    if new_content != content:
        if "BBOT_TESTING" not in os.environ:
            with open(file, "w") as f:
                f.write(new_content)


def update_docs():
    md_files = [p for p in bbot_code_dir.glob("**/*.md") if p.is_file()]

    def update_md_files(keyword, s):
        for file in md_files:
            find_replace_file(file, keyword, s)

    def update_individual_module_options():
        regex = re.compile("BBOT MODULE OPTIONS ([A-Z_]+)")
        for file in md_files:
            with open(file) as f:
                content = f.read()
            for match in regex.finditer(content):
                module_name = match.groups()[0].lower()
                bbot_module_options_table = DEFAULT_PRESET.module_loader.modules_options_table(modules=[module_name])
                find_replace_file(file, f"BBOT MODULE OPTIONS {module_name.upper()}", bbot_module_options_table)

    # Example commands
    bbot_example_commands = []
    for title, description, command in DEFAULT_PRESET.args.scan_examples:
        example = ""
        example += f"**{title}:**\n\n"
        # example += f"{description}\n"
        example += f"```bash\n# {description}\n{command}\n```"
        bbot_example_commands.append(example)
    bbot_example_commands = "\n\n".join(bbot_example_commands)
    assert len(bbot_example_commands.splitlines()) > 10
    update_md_files("BBOT EXAMPLE COMMANDS", bbot_example_commands)

    # Help output
    bbot_help_output = DEFAULT_PRESET.args.parser.format_help().replace("docs.py", "bbot")
    bbot_help_output = f"```text\n{bbot_help_output}\n```"
    assert len(bbot_help_output.splitlines()) > 50
    update_md_files("BBOT HELP OUTPUT", bbot_help_output)

    # BBOT events
    bbot_event_table = DEFAULT_PRESET.module_loader.events_table()
    assert len(bbot_event_table.splitlines()) > 10
    update_md_files("BBOT EVENTS", bbot_event_table)

    # BBOT modules
    bbot_module_table = DEFAULT_PRESET.module_loader.modules_table(include_author=True, include_created_date=True)
    assert len(bbot_module_table.splitlines()) > 50
    update_md_files("BBOT MODULES", bbot_module_table)

    # BBOT output modules
    bbot_output_module_table = DEFAULT_PRESET.module_loader.modules_table(
        mod_type="output", include_author=True, include_created_date=True
    )
    assert len(bbot_output_module_table.splitlines()) > 10
    update_md_files("BBOT OUTPUT MODULES", bbot_output_module_table)

    # BBOT universal module options
    from bbot.scanner.preset.args import universal_module_options

    universal_module_options_table = ""
    for option, description in universal_module_options.items():
        universal_module_options_table += f"**{option}**: {description}\n"
    update_md_files("BBOT UNIVERSAL MODULE OPTIONS", universal_module_options_table)

    # BBOT module options
    bbot_module_options_table = DEFAULT_PRESET.module_loader.modules_options_table()
    assert len(bbot_module_options_table.splitlines()) > 100
    update_md_files("BBOT MODULE OPTIONS", bbot_module_options_table)
    update_individual_module_options()

    # BBOT module flags
    bbot_module_flags_table = DEFAULT_PRESET.module_loader.flags_table()
    assert len(bbot_module_flags_table.splitlines()) > 10
    update_md_files("BBOT MODULE FLAGS", bbot_module_flags_table)

    # BBOT presets
    bbot_presets_table = DEFAULT_PRESET.presets_table(include_modules=True)
    assert len(bbot_presets_table.splitlines()) > 5
    update_md_files("BBOT PRESETS", bbot_presets_table)

    # BBOT presets
    for _, (loaded_preset, category, preset_path, original_filename) in DEFAULT_PRESET.all_presets.items():
        str_category = "" if not category else f"/{category}"
        filename = f"~/.bbot/presets{str_category}/{original_filename.name}"
        preset_yaml = f"""
```yaml title={filename}
{loaded_preset._yaml_str}
```
"""
        preset_yaml_expandable = f"""
<details>
<summary><b><code>{original_filename.name}</code></b></summary>

```yaml
{loaded_preset._yaml_str}
```

</details>
"""
        update_md_files(f"BBOT {loaded_preset.name.upper()} PRESET", preset_yaml)
        update_md_files(f"BBOT {loaded_preset.name.upper()} PRESET EXPANDABLE", preset_yaml_expandable)

    content = []
    for _, (loaded_preset, category, preset_path, original_filename) in DEFAULT_PRESET.all_presets.items():
        yaml_str = loaded_preset._yaml_str
        indent = " " * 4
        yaml_str = f"\n{indent}".join(yaml_str.splitlines())
        str_category = "" if not category else f"/{category}"
        filename = f"~/.bbot/presets{str_category}/{original_filename.name}"

        num_modules = len(loaded_preset.scan_modules)
        modules = ", ".join(sorted([f"`{m}`" for m in loaded_preset.scan_modules]))
        category = f"Category: {category}" if category else ""

        content.append(
            f"""## **{loaded_preset.name}**

{loaded_preset.description}

??? note "`{original_filename.name}`"
    ```yaml title="{filename}"
    {yaml_str}
    ```

{category}

Modules: [{num_modules:,}]("{modules}")"""
        )
    assert len(content) > 5
    update_md_files("BBOT PRESET YAML", "\n\n".join(content))

    # Default config
    default_config_file = bbot_code_dir / "bbot" / "defaults.yml"
    with open(default_config_file) as f:
        default_config_yml = f.read()
    default_config_yml = f'```yaml title="defaults.yml"\n{default_config_yml}\n```'
    assert len(default_config_yml.splitlines()) > 20
    update_md_files("BBOT DEFAULT CONFIG", default_config_yml)

    # Table of Contents
    base_url = "https://www.blacklanternsecurity.com/bbot/Stable"

    def format_section(section_title, section_path):
        path = section_path.split("index.md")[0]
        path = path.split(".md")[0]
        return f"- [{section_title}]({base_url}/{path})\n"

    bbot_docs_toc = ""

    def update_toc(section, level=0):
        nonlocal bbot_docs_toc
        indent = " " * 4 * level
        if isinstance(section, dict):
            for section_title, subsections in section.items():
                if isinstance(subsections, str):
                    bbot_docs_toc += f"{indent}{format_section(section_title, subsections)}"
                else:
                    bbot_docs_toc += f"{indent}- **{section_title}**\n"
                    for subsection in subsections:
                        update_toc(subsection, level=level + 1)

    mkdocs_yml_file = bbot_code_dir / "mkdocs.yml"
    yaml.SafeLoader.add_constructor(
        "tag:yaml.org,2002:python/name:pymdownx.superfences.fence_code_format", lambda x, y: {}
    )

    with open(mkdocs_yml_file, "r") as f:
        mkdocs_yaml = yaml.safe_load(f)
        nav = mkdocs_yaml["nav"]
        for section in nav:
            update_toc(section)
    bbot_docs_toc = bbot_docs_toc.strip()
    # assert len(bbot_docs_toc.splitlines()) == 2
    update_md_files("BBOT DOCS TOC", bbot_docs_toc)

    # generate data for chord graph
    gen_chord_data()


update_docs()



import importlib
from bbot.modules.base import BaseModule

from bbot.errors import InteractshError


class lightfuzz(BaseModule):
    watched_events = ["URL", "WEB_PARAMETER"]
    produced_events = ["FINDING", "VULNERABILITY"]
    flags = ["active", "aggressive", "web-thorough", "deadly"]

    options = {
        "force_common_headers": False,
        "enabled_submodules": ["sqli", "cmdi", "xss", "path", "ssti", "crypto", "serial"],
        "disable_post": False,
    }
    options_desc = {
        "force_common_headers": "Force emit commonly exploitable parameters that may be difficult to detect",
        "enabled_submodules": "A list of submodules to enable. Empty list enabled all modules.",
        "disable_post": "Disable processing of POST parameters, avoiding form submissions.",
    }

    meta = {
        "description": "Find Web Parameters and Lightly Fuzz them using a heuristic based scanner",
        "author": "@liquidsec",
        "created_date": "2024-06-28",
    }
    common_headers = ["x-forwarded-for", "user-agent"]
    in_scope_only = True

    _module_threads = 4

    async def setup(self):
        self.event_dict = {}
        self.interactsh_subdomain_tags = {}
        self.interactsh_instance = None
        self.disable_post = self.config.get("disable_post", False)
        self.enabled_submodules = self.config.get("enabled_submodules")
        self.interactsh_disable = self.scan.config.get("interactsh_disable", False)
        self.submodules = {}

        if not self.enabled_submodules:
            return False, "Lightfuzz enabled without any submodules. Must enable at least one submodule."

        for submodule_name in self.enabled_submodules:
            try:
                submodule_module = importlib.import_module(f"bbot.modules.lightfuzz.submodules.{submodule_name}")
                submodule_class = getattr(submodule_module, submodule_name)
            except ImportError:
                return False, f"Invalid Lightfuzz submodule ({submodule_name}) specified in enabled_modules"
            self.submodules[submodule_name] = submodule_class

        interactsh_needed = any(submodule.uses_interactsh for submodule in self.submodules.values())

        if interactsh_needed and not self.interactsh_disable:
            try:
                self.interactsh_instance = self.helpers.interactsh()
                self.interactsh_domain = await self.interactsh_instance.register(callback=self.interactsh_callback)
            except InteractshError as e:
                self.warning(f"Interactsh failure: {e}")
        return True

    async def interactsh_callback(self, r):
        full_id = r.get("full-id", None)
        if full_id:
            if "." in full_id:
                details = self.interactsh_subdomain_tags.get(full_id.split(".")[0])
                if not details["event"]:
                    return
                # currently, this is only used by the cmdi submodule. Later, when other modules use it, we will need to store description data in the interactsh_subdomain_tags dictionary
                await self.emit_event(
                    {
                        "severity": "CRITICAL",
                        "host": str(details["event"].host),
                        "url": details["event"].data["url"],
                        "description": f"OS Command Injection (OOB Interaction) Type: [{details['type']}] Parameter Name: [{details['name']}] Probe: [{details['probe']}]",
                    },
                    "VULNERABILITY",
                    details["event"],
                )
            else:
                # this is likely caused by something trying to resolve the base domain first and can be ignored
                self.debug("skipping result because subdomain tag was missing")

    def _outgoing_dedup_hash(self, event):
        return hash(
            (
                "lightfuzz",
                str(event.host),
                event.data["url"],
                event.data["description"],
                event.data.get("type", ""),
                event.data.get("name", ""),
            )
        )

    async def run_submodule(self, submodule, event):
        submodule_instance = submodule(self, event)
        await submodule_instance.fuzz()
        if len(submodule_instance.results) > 0:
            for r in submodule_instance.results:
                event_data = {"host": str(event.host), "url": event.data["url"], "description": r["description"]}

                envelopes = getattr(event, "envelopes", None)
                envelope_summary = getattr(envelopes, "summary", None)
                if envelope_summary:
                    # Append the envelope summary to the description
                    event_data["description"] += f" Envelopes: [{envelope_summary}]"

                if r["type"] == "VULNERABILITY":
                    event_data["severity"] = r["severity"]
                await self.emit_event(
                    event_data,
                    r["type"],
                    event,
                )

    async def handle_event(self, event):
        if event.type == "URL":
            if self.config.get("force_common_headers", False) is False:
                return False

            # If force_common_headers is True, we force the emission of a WEB_PARAMETER for each of the common headers to force fuzzing against them
            for h in self.common_headers:
                description = f"Speculative (Forced) Header [{h}]"
                data = {
                    "host": str(event.host),
                    "type": "HEADER",
                    "name": h,
                    "original_value": None,
                    "url": event.data,
                    "description": description,
                }
                await self.emit_event(data, "WEB_PARAMETER", event)

        elif event.type == "WEB_PARAMETER":
            # check connectivity to url
            connectivity_test = await self.helpers.request(event.data["url"], timeout=10)

            if connectivity_test:
                for submodule_name, submodule in self.submodules.items():
                    self.debug(f"Starting {submodule_name} fuzz()")
                    await self.run_submodule(submodule, event)
            else:
                self.debug(f"WEB_PARAMETER URL {event.data['url']} failed connectivity test, aborting")

    async def cleanup(self):
        if self.interactsh_instance:
            try:
                await self.interactsh_instance.deregister()
                self.debug(
                    f"successfully deregistered interactsh session with correlation_id {self.interactsh_instance.correlation_id}"
                )
            except InteractshError as e:
                self.warning(f"Interactsh failure: {e}")

    async def finish(self):
        if self.interactsh_instance:
            await self.helpers.sleep(5)
            try:
                for r in await self.interactsh_instance.poll():
                    await self.interactsh_callback(r)
            except InteractshError as e:
                self.debug(f"Error in interact.sh: {e}")

    # If we've disabled fuzzing POST parameters, back out of POSTPARAM WEB_PARAMETER events as quickly as possible
    async def filter_event(self, event):
        if event.type == "WEB_PARAMETER" and self.disable_post and event.data["type"] == "POSTPARAM":
            return False, "POST parameter disabled in lightfuzz module"
        return True

    @classmethod
    def help_text(self):
        # Call the base class help_text method
        base_help_text = super().help_text()

        import importlib

        submodules = {}
        for submodule_name in self.options.get("enabled_submodules", []):
            try:
                submodule_module = importlib.import_module(f"bbot.modules.lightfuzz.submodules.{submodule_name}")
                submodule_class = getattr(submodule_module, submodule_name)
                submodules[submodule_name] = submodule_class
            except ImportError:
                continue

        # Find all submodules
        submodules_info = "\nLightfuzz Submodules:\n"
        for submodule_name, submodule_class in submodules.items():
            try:
                friendly_name = getattr(submodule_class, "friendly_name", submodule_name)
                description = (
                    submodule_class.__doc__.strip() if submodule_class.__doc__ else "No description available"
                )
                indented_description = "      " + description.replace("\n", "\n      ")
                submodules_info += f"  - {submodule_name} ({friendly_name}):\n"
                submodules_info += f"{indented_description}\n\n"
            except AttributeError:
                continue

        # Combine the base help text with the submodules information
        return base_help_text + submodules_info



from .base import ModuleTestBase


class TestSubDomainRadar(ModuleTestBase):
    config_overrides = {"modules": {"subdomainradar": {"api_key": "asdf"}}}

    async def setup_before_prep(self, module_test):
        await module_test.mock_dns(
            {
                "blacklanternsecurity.com": {"A": ["127.0.0.88"]},
                "www.blacklanternsecurity.com": {"A": ["127.0.0.88"]},
                "asdf.blacklanternsecurity.com": {"A": ["127.0.0.88"]},
            }
        )
        module_test.httpx_mock.add_response(
            url="https://api.subdomainradar.io/profile",
            match_headers={"Authorization": "Bearer asdf"},
        )
        module_test.httpx_mock.add_response(
            url="https://api.subdomainradar.io/enumerate",
            method="POST",
            json={
                "tasks": {"blacklanternsecurity.com": "86de4531-0a67-41fe-b5e4-8ce8207d6245"},
                "message": "Tasks initiated",
            },
            match_headers={"Authorization": "Bearer asdf"},
        )
        module_test.httpx_mock.add_response(
            url="https://api.subdomainradar.io/tasks/86de4531-0a67-41fe-b5e4-8ce8207d6245",
            match_headers={"Authorization": "Bearer asdf"},
            json={
                "task_id": "86de4531-0a67-41fe-b5e4-8ce8207d6245",
                "status": "completed",
                "domain": "blacklanternsecurity.com",
                "subdomains": [
                    {
                        "subdomain": "www.blacklanternsecurity.com",
                        "ip": None,
                        "reverse_dns": [],
                        "country": None,
                        "timestamp": None,
                    },
                    {
                        "subdomain": "asdf.blacklanternsecurity.com",
                        "ip": None,
                        "reverse_dns": [],
                        "country": None,
                        "timestamp": None,
                    },
                ],
                "total_subdomains": 2,
                "rank": None,
                "whois": {
                    "domain_name": ["BLACKLANTERNSECURITY.COM", "blacklanternsecurity.com"],
                    "registrar": "MarkMonitor, Inc.",
                    "creation_date": ["1992-11-04T05:00:00", "1992-11-04T05:00:00+00:00"],
                    "expiration_date": ["2026-11-03T05:00:00", "2026-11-03T00:00:00+00:00"],
                    "last_updated": ["2024-10-02T10:15:20", "2024-10-02T10:15:20+00:00"],
                    "status": [
                        "clientDeleteProhibited https://icann.org/epp#clientDeleteProhibited",
                        "clientTransferProhibited https://icann.org/epp#clientTransferProhibited",
                        "clientUpdateProhibited https://icann.org/epp#clientUpdateProhibited",
                        "serverDeleteProhibited https://icann.org/epp#serverDeleteProhibited",
                        "serverTransferProhibited https://icann.org/epp#serverTransferProhibited",
                        "serverUpdateProhibited https://icann.org/epp#serverUpdateProhibited",
                        "clientUpdateProhibited (https://www.icann.org/epp#clientUpdateProhibited)",
                        "clientTransferProhibited (https://www.icann.org/epp#clientTransferProhibited)",
                        "clientDeleteProhibited (https://www.icann.org/epp#clientDeleteProhibited)",
                        "serverUpdateProhibited (https://www.icann.org/epp#serverUpdateProhibited)",
                        "serverTransferProhibited (https://www.icann.org/epp#serverTransferProhibited)",
                        "serverDeleteProhibited (https://www.icann.org/epp#serverDeleteProhibited)",
                    ],
                    "nameservers": [
                        "A1-12.AKAM.NET",
                        "A10-67.AKAM.NET",
                        "A12-64.AKAM.NET",
                        "A28-65.AKAM.NET",
                        "A7-66.AKAM.NET",
                        "A9-67.AKAM.NET",
                        "EDNS69.ULTRADNS.BIZ",
                        "EDNS69.ULTRADNS.COM",
                        "EDNS69.ULTRADNS.NET",
                        "EDNS69.ULTRADNS.ORG",
                        "edns69.ultradns.biz",
                        "a12-64.akam.net",
                        "edns69.ultradns.net",
                        "edns69.ultradns.org",
                        "a10-67.akam.net",
                        "a28-65.akam.net",
                        "a9-67.akam.net",
                        "a1-12.akam.net",
                        "a7-66.akam.net",
                        "edns69.ultradns.com",
                    ],
                    "emails": [
                        "abusecomplaints@markmonitor.com",
                        "admin@dnstinations.com",
                        "whoisrequest@markmonitor.com",
                    ],
                    "dnssec": "unsigned",
                    "org": "DNStination Inc.",
                    "address": "3450 Sacramento Street, Suite 405",
                    "city": "San Francisco",
                    "state": "CA",
                    "zipcode": None,
                    "country": "US",
                },
                "enumerators": ["Aquarius Enumerator", "Beta Enumerator", "Chi Enumerator", "Eta Enumerator"],
                "timestamp": "2024-10-06T02:48:10.075636",
                "error": None,
                "is_notification": False,
                "notification_domain_id": None,
                "demo": False,
                "user_id": 49,
                "time_to_finish": 41,
            },
        )
        module_test.httpx_mock.add_response(
            url="https://api.subdomainradar.io/enumerators/groups",
            match_headers={"Authorization": "Bearer asdf"},
            json=[
                {
                    "id": "1",
                    "name": "Fast",
                    "description": "Enumerators optimized for high-speed scanning and rapid data collection",
                    "enumerators": [
                        {"display_name": "Beta Enumerator"},
                        {"display_name": "Chi Enumerator"},
                        {"display_name": "Aquarius Enumerator"},
                        {"display_name": "Eta Enumerator"},
                    ],
                },
                {
                    "id": "2",
                    "name": "Medium",
                    "description": "Enumerators balanced for moderate speed with a focus on thoroughness",
                    "enumerators": [
                        {"display_name": "Kappa Enumerator"},
                        {"display_name": "Lambda Enumerator"},
                        {"display_name": "Mu Enumerator"},
                        {"display_name": "Pi Enumerator"},
                        {"display_name": "Tau Enumerator"},
                        {"display_name": "Beta Enumerator"},
                        {"display_name": "Chi Enumerator"},
                        {"display_name": "Psi Enumerator"},
                        {"display_name": "Aquarius Enumerator"},
                        {"display_name": "Zeta Enumerator"},
                        {"display_name": "Eta Enumerator"},
                    ],
                },
                {
                    "id": "3",
                    "name": "Deep",
                    "description": "Enumerators designed for exhaustive searches and in-depth data analysis",
                    "enumerators": [
                        {"display_name": "Alpha Enumerator"},
                        {"display_name": "Kappa Enumerator"},
                        {"display_name": "Lambda Enumerator"},
                        {"display_name": "Mu Enumerator"},
                        {"display_name": "Nu Enumerator"},
                        {"display_name": "Xi Enumerator"},
                        {"display_name": "Pi Enumerator"},
                        {"display_name": "Rho Enumerator"},
                        {"display_name": "Sigma Enumerator"},
                        {"display_name": "Tau Enumerator"},
                        {"display_name": "Beta Enumerator"},
                        {"display_name": "Chi Enumerator"},
                        {"display_name": "Omega Enumerator"},
                        {"display_name": "Psi Enumerator"},
                        {"display_name": "Phi Enumerator"},
                        {"display_name": "Axon Enumerator"},
                        {"display_name": "Aquarius Enumerator"},
                        {"display_name": "Pegasus Enumerator"},
                        {"display_name": "Petra Enumerator"},
                        {"display_name": "Oasis Enumerator"},
                        {"display_name": "Mike Enumerator"},
                        {"display_name": "Cat Enumerator"},
                        {"display_name": "Brutus Enumerator"},
                        {"display_name": "Dee Enumerator"},
                        {"display_name": "Jul Enumerator"},
                        {"display_name": "Eve Enumerator"},
                        {"display_name": "Frank Enumerator"},
                        {"display_name": "Gus Enumerator"},
                        {"display_name": "Hank Enumerator"},
                        {"display_name": "Delta Enumerator"},
                        {"display_name": "Ivy Enumerator"},
                        {"display_name": "Jack Enumerator"},
                        {"display_name": "Karl Enumerator"},
                        {"display_name": "Liam Enumerator"},
                        {"display_name": "Nora Enumerator"},
                        {"display_name": "Mars Enumerator"},
                        {"display_name": "Neptune Enumerator"},
                        {"display_name": "Orion Enumerator"},
                        {"display_name": "Oedipus Enumerator"},
                        {"display_name": "Pandora Enumerator"},
                        {"display_name": "Epsilon Enumerator"},
                        {"display_name": "Zeta Enumerator"},
                        {"display_name": "Eta Enumerator"},
                        {"display_name": "Theta Enumerator"},
                        {"display_name": "Iota Enumerator"},
                    ],
                },
            ],
        )

    def check(self, module_test, events):
        assert any(e.data == "www.blacklanternsecurity.com" for e in events), "Failed to detect subdomain #1"
        assert any(e.data == "asdf.blacklanternsecurity.com" for e in events), "Failed to detect subdomain #2"



import asyncio
import time

from .base import ModuleTestBase


class TestMySQL(ModuleTestBase):
    targets = ["evilcorp.com"]
    skip_distro_tests = True

    async def setup_before_prep(self, module_test):
        process = await asyncio.create_subprocess_exec(
            "docker",
            "run",
            "--name",
            "bbot-test-mysql",
            "--rm",
            "-e",
            "MYSQL_ROOT_PASSWORD=bbotislife",
            "-e",
            "MYSQL_DATABASE=bbot",
            "-p",
            "3306:3306",
            "-d",
            "mysql",
            stdout=asyncio.subprocess.PIPE,
            stderr=asyncio.subprocess.PIPE,
        )
        stdout, stderr = await process.communicate()

        import aiomysql

        # wait for the container to start
        start_time = time.time()
        while True:
            try:
                conn = await aiomysql.connect(user="root", password="bbotislife", db="bbot", host="localhost")
                conn.close()
                break
            except Exception as e:
                if time.time() - start_time > 60:  # timeout after 60 seconds
                    self.log.error("MySQL server did not start in time.")
                    raise e
                await asyncio.sleep(1)

        if process.returncode != 0:
            self.log.error(f"Failed to start MySQL server: {stderr.decode()}")

    async def check(self, module_test, events):
        import aiomysql

        # Connect to the MySQL database
        conn = await aiomysql.connect(user="root", password="bbotislife", db="bbot", host="localhost")

        try:
            async with conn.cursor() as cur:
                await cur.execute("SELECT * FROM event")
                events = await cur.fetchall()
                assert len(events) == 3, "No events found in MySQL database"

                await cur.execute("SELECT * FROM scan")
                scans = await cur.fetchall()
                assert len(scans) == 1, "No scans found in MySQL database"

                await cur.execute("SELECT * FROM target")
                targets = await cur.fetchall()
                assert len(targets) == 1, "No targets found in MySQL database"
        finally:
            conn.close()
            process = await asyncio.create_subprocess_exec(
                "docker", "stop", "bbot-test-mysql", stdout=asyncio.subprocess.PIPE, stderr=asyncio.subprocess.PIPE
            )
            stdout, stderr = await process.communicate()

            if process.returncode != 0:
                raise Exception(f"Failed to stop MySQL server: {stderr.decode()}")
